"""Autocomplete."""

class Trie:
    """A Trie.

    >>> t = Trie()
    >>> t.add("cat")
    >>> t.add("dog")
    >>> t.add("dot")
    >>> t.lookup("")
    ['cat', 'dog', 'dot']
    >>> t.lookup("d")
    ['dog', 'dot']
    >>> t.lookup("do")
    ['dog', 'dot']
    >>> t.lookup("dog")
    ['dog']
    """
    def __init__(self):
        self.word = None
        self.letters = {}

    def add(self, word, suffix=None):
        """Add word to self, indexed by suffix."""
        if suffix is None:
            suffix = word
        if not suffix:
            self.word = word
        else:
            first, rest = suffix[0], suffix[1:]
            if first not in self.letters:
                self.letters[first] = Trie()
            self.letters[first].add(word, rest)

    def lookup(self, prefix):
        """Lookup all words matching prefix."""
        if not prefix:
            branches = self.letters.values()
            words = sum([b.lookup(prefix) for b in branches], [])
            if self.word:
                words.append(self.word)
            return sorted(words)
        else:
            first, rest = prefix[0], prefix[1:]
            if first not in self.letters:
                return []
            else:
                return self.letters[first].lookup(rest)

def build_trie(words):
    trie = Trie()
    for word in words:
        word = word.strip().lower()
        trie.add(word)
    return trie

trie = build_trie(open("/usr/share/dict/words").readlines())

import flask
app = flask.Flask(__name__)

@app.route("/")
def main():
    return open('words.html').read()

@app.route("/extend/<prefix>")
def extend(prefix):
    words = trie.lookup(prefix)[:10]
    return flask.jsonify({'words': words})

import requests

@app.route("/hello/<name>")
def hello(name):
    r = requests.get('http://en.wikipedia.org/wiki/' + name)
    return r.text

if __name__ == "__main__":
    app.run(debug=True)
